package Win32::LangId;

#-- LangIds

our %LangId = (
    Afrikaans => {SouthAfrica => 0x0436,},
    Albanian  => {Albania     => 0x041c,},
    Alsatian  => {France      => 0x0484,},
    Amharic   => {Ethiopia    => 0x045e,},
    Arabic    => {
        Algeria     => 0x1401,
        Bahrain     => 0x3c01,
        Egypt       => 0x0c01,
        Iraq        => 0x0801,
        Jordan      => 0x2c01,
        Kuwait      => 0x3401,
        Lebanon     => 0x3001,
        Libya       => 0x1001,
        Morocco     => 0x1801,
        Oman        => 0x2001,
        Qatar       => 0x4001,
        SaudiArabia => 0x0401,
        Syria       => 0x2801,
        Tunisia     => 0x1c01,
        UAE         => 0x3801,
        Yemen       => 0x2401,
        },
    Armenian => {Armenia            => 0x042b,},
    Assamese => {India              => 0x044d,},
    Azeri    => {AzerbaijanCyrillic => 0x082c, AzerbaijanLatin => 0x042c,},
    Bashkir  => {Russia             => 0x046d,},
    Basque   => {Basque             => 0x042d,},
    Belarusian => {Belarus => 0x0423, Bengali => 0x0445, Bosnian => 0x781a,},
    Bosnian => {
        BosniaandHerzegovinaCyrillic => 0x201a,
        BosniaandHerzegovinaLatin    => 0x141a,
        },
    Breton    => {France   => 0x047e,},
    Bulgarian => {Bulgaria => 0x0402,},
    Catalan   => {Catalan  => 0x0403,},
    Chinese   => {
        HongKong    => 0x0c04,
        Macao       => 0x1404,
        Singapore   => 0x1004,
        Simplified  => 0x0804,
        Traditional => 0x0404,
        },
    Corsican => {France                    => 0x0483, Croatian    => 0x041a,},
    Croatian => {BosniaandHerzegovinaLatin => 0x101a, Croatia     => 0x041a,},
    Czech    => {CzechRepublic             => 0x0405,},
    Danish   => {Denmark                   => 0x0406,},
    Dari     => {Afghanistan               => 0x048c,},
    Divehi   => {Maldives                  => 0x0465,},
    Dutch    => {Belgium                   => 0x0813, Netherlands => 0x0413,},
    English  => {
        Australia         => 0x0c09,
        Belize            => 0x2809,
        Canada            => 0x1009,
        Caribbean         => 0x2409,
        India             => 0x4009,
        Ireland           => 0x1809,
        Ireland           => 0x1809,
        Jamaica           => 0x2009,
        Malaysia          => 0x4409,
        NewZealand        => 0x1409,
        Philippines       => 0x3409,
        Singapore         => 0x4809,
        SouthAfrica       => 0x1c09,
        TrinidadandTobago => 0x2c09,
        UnitedKingdom     => 0x0809,
        UnitedStates      => 0x0409,
        Zimbabwe          => 0x3009,
        },
    Estonian => {Estonia      => 0x0425,},
    Faroese  => {FaroeIslands => 0x0438,},
    Filipino => {Philippines  => 0x0464,},
    Finnish  => {Finland      => 0x040b,},
    French   => {
        Belgium     => 0x080c,
        Canada      => 0x0c0c,
        France      => 0x040c,
        Luxembourg  => 0x140c,
        Monaco      => 0x180c,
        Switzerland => 0x100c,
        },
    Frisian  => {Netherlands => 0x0462,},
    Galician => {Spain       => 0x0456,},
    Georgian => {Georgia     => 0x0437,},
    German   => {
        Austria       => 0x0c07,
        Germany       => 0x0407,
        Liechtenstein => 0x1407,
        Luxembourg    => 0x1007,
        Switzerland   => 0x0807,
        },
    Greek         => {Greece           => 0x0408,},
    Greenlandic   => {Greenland        => 0x046f,},
    Gujarati      => {India            => 0x0447,},
    Hausa         => {Nigeria          => 0x0468,},
    Hebrew        => {Israel           => 0x040d,},
    Hindi         => {India            => 0x0439,},
    Hungarian     => {Hungary          => 0x040e,},
    Icelandic     => {Iceland          => 0x040f,},
    Igbo          => {Nigeria          => 0x0470,},
    Indonesian    => {Indonesia        => 0x0421,},
    Inuktitut     => {Canada           => 0x085d, Canada => 0x045d,},
    Irish         => {Ireland          => 0x083c,},
    Italian       => {Italy            => 0x0410, Switzerland => 0x0810,},
    Japanese      => {Japan            => 0x0411,},
    Kannada       => {India            => 0x044b,},
    Kazakh        => {Kazakhstan       => 0x043f,},
    Khmer         => {Cambodia         => 0x0453,},
    "K'iche"      => {Guatemala        => 0x0486,},
    Kinyarwanda   => {Rwanda           => 0x0487,},
    Konkani       => {India            => 0x0457,},
    Korean        => {Korea            => 0x0412,},
    Kyrgyz        => {Kyrgyzstan       => 0x0440,},
    Lao           => {LaoPDR           => 0x0454,},
    Latvian       => {Latvia           => 0x0426,},
    Lithuanian    => {Lithuanian       => 0x0427,},
    LowerSorbian  => {Germany          => 0x082e,},
    Luxembourgish => {Luxembourg       => 0x046e,},
    Macedonian    => {MacedoniaFYROM   => 0x042f,},
    Malay         => {BruneiDarassalam => 0x083e, Malaysia => 0x043e,},
    Malayalam     => {India            => 0x044c,},
    Maltese       => {Malta            => 0x043a,},
    Maori         => {NewZealand       => 0x0481,},
    Mapudungun    => {Chile            => 0x047a,},
    Marathi       => {India            => 0x044e,},
    Mohawk        => {Canada           => 0x047c,},
    Mongolian     => {MongoliaCyrillic => 0x0450, Mongolia => 0x0850,},
    Nepali        => {Nepal            => 0x0461,},
    Norwegian     => {BokmlNorway      => 0x0414, NynorskNorway => 0x0814,},
    Occitan       => {France           => 0x0482,},
    Oriya         => {India            => 0x0448,},
    Pashto        => {Afghanistan      => 0x0463,},
    Persian       => {Iran             => 0x0429,},
    Polish        => {Poland           => 0x0415,},
    Portuguese => {Brazil => 0x0416, Portugal => 0x0816, Portugal => 0x0816,},
    Punjabi    => {India  => 0x0446,},
    Quechua  => {Bolivia     => 0x046b, Ecuador => 0x086b, Peru => 0x0c6b,},
    Romanian => {Romania     => 0x0418,},
    Romansh  => {Switzerland => 0x0417,},
    Russian  => {Russia      => 0x0419,},
    Sami     => {
        InariFinland    => 0x243b,
        LuleNorway      => 0x103b,
        LuleSweden      => 0x143b,
        NorthernFinland => 0x0c3b,
        NorthernNorway  => 0x043b,
        NorthernSweden  => 0x083b,
        SkoltFinland    => 0x203b,
        SouthernNorway  => 0x183b,
        SouthernSweden  => 0x1c3b,
        },
    Sanskrit => {
        India                     => 0x044f,
        Serbian                   => 0x7c1a,
        BosniaandHerzegovinaLatin => 0x181a,
        SerbiaCyrillic            => 0x0c1a,
        SerbiaLatin               => 0x081a,
        },
    SesothosaLeboa_NorthernSotho => {SouthAfrica => 0x046c,},
    Setswana_Tswana              => {SouthAfrica => 0x0432,},
    Sinhala                      => {SriLanka    => 0x045b,},
    Slovak                       => {Slovakia    => 0x041b,},
    Slovenian                    => {Slovenia    => 0x0424,},
    Spanish                      => {
        Argentina         => 0x2c0a,
        Bolivia           => 0x400a,
        Chile             => 0x340a,
        Colombia          => 0x240a,
        CostaRica         => 0x140a,
        DominicanRepublic => 0x1c0a,
        Ecuador           => 0x300a,
        ElSalvador        => 0x440a,
        Guatemala         => 0x100a,
        Honduras          => 0x480a,
        Mexico            => 0x080a,
        Nicaragua         => 0x4c0a,
        Panama            => 0x180a,
        Paraguay          => 0x3c0a,
        Peru              => 0x280a,
        PuertoRico        => 0x500a,
        Spain             => 0x0c0a,
        Uruguay           => 0x380a,
        Venezuela         => 0x200a,
        },
    Swahili   => {Kenya        => 0x0441,},
    Swedish   => {Finland      => 0x081d, Sweden => 0x041d, Sweden => 0x041d,},
    Syriac    => {Syria        => 0x045a,},
    Tajik     => {Tajikistan   => 0x0428,},
    Tamazight => {AlgeriaLatin => 0x085f,},
    Tamil     => {India        => 0x0449,},
    Tatar     => {Russia       => 0x0444,},
    Telugu    => {India        => 0x044a,},
    Thai      => {Thailand     => 0x041e,},
    Tibetan   => {PRC          => 0x0451,},
    Turkish   => {Turkey       => 0x041f,},
    Turkmen   => {Turkmenistan => 0x0442,},
    Uighur    => {PRC          => 0x0480,},
    Ukrainian => {Ukraine      => 0x0422,},
    UpperSorbian => {Germany            => 0x042e,},
    Urdu         => {reserved           => 0x0820, Pakistan => 0x0420,},
    Uzbek        => {UzbekistanCyrillic => 0x0843, UzbekistanLatin => 0x0443,},
    Vietnamese   => {Vietnam            => 0x042a,},
    Welsh        => {UnitedKingdom      => 0x0452,},
    Wolof        => {Senegal            => 0x0488,},
    Xhosa_isiXhosa => {SouthAfrica => 0x0434,},
    Yakut          => {Russia      => 0x0485,},
    Yi             => {PRC         => 0x0478,},
    Yoruba         => {Nigeria     => 0x046a,},
    Zulu_isiZulu   => {SouthAfrica => 0x0435,},
    );

1;
